/*
 * File:		arp.h
 * Purpose:		ARP definitions.
 *
 * Notes:
 *
 * Date:		3-26-96
 *
 * Modifications:
 *
 */


#ifndef _ARP_H
#define _ARP_H


#include "eth.h"
#include "nif.h"

/*
 * This data definition is defined for Ethernet only!
 */
typedef struct
{
	UINT16		ar_hrd;
	UINT16		ar_pro;
	UINT8		ar_hln;
	UINT8		ar_pln;
	UINT16		opcode;
	UINT8		ar_sha[6];	/* ethernet hw address */
	UINT8		ar_spa[4];	/* ip address */
	UINT8		ar_tha[6];	/* ethernet hw address */
	UINT8		ar_tpa[4];	/* ip address */
} arp_frame_hdr;

/*
 * ARP table entry definition.  Note that this table only designed
 * with Ethernet and IP in mind.
 */
#define MAX_HWA_SIZE	(6)	/* 6 is enough for Ethernet address */
#define MAX_PA_SIZE		(4)	/* 4 is enough for Protocol address */
typedef struct
{
	UINT16	proto;
	UINT8	hwa_size;
	UINT8	hwa[MAX_HWA_SIZE];
	UINT8	pa_size;
	UINT8	pa[MAX_PA_SIZE];
	int		longevity;
} ARPENTRY;
#define MAX_ARP_ENTRY	(10)

typedef struct
{
	unsigned int	tab_size;
	ARPENTRY		table[MAX_ARP_ENTRY];
} PROTO_ARP_INFO;

#define ARP_ENTRY_EMPTY	(0)
#define ARP_ENTRY_PERM	(1)
#define ARP_ENTRY_TEMP	(2)


#define ETHERNET		(1)
#define	ARP_REQUEST		(1)
#define	ARP_REPLY		(2)

/****************************************************************/
UINT8 *
arp_get_mypa (void);

UINT8 *
arp_get_myha (void);

UINT8 *
arp_get_broadcast (void);

void
arp_merge (PROTO_ARP_INFO *, UINT16, int, UINT8 *, int, UINT8 *, int);

void
arp_remove (PROTO_ARP_INFO *, UINT16, UINT8 *, UINT8 *);

void
arp_request (NIF *, UINT8 *);

int
arp_handler (NIF *, arp_frame_hdr *, int frame_length);

UINT8 *
arp_resolve (NIF *, UINT16, UINT8 *);

void
proto_arp_config (PROTO_ARP_INFO *);

/****************************************************************/

#endif	/* _ARP_H */
