/*
 * File:		ip.h
 * Purpose:		Definitions for the Internet Protocol, IP.
 *
 * Notes:		See RFC 791 "DARPA Internet Program Protocol
 *				Specification" for more details.
 *
 * Date:		11-8-95
 *
 * Modifications:
 *
 */

#ifndef _IP_H
#define _IP_H

#include "nif.h"

/* 32-bit IP Addresses */
typedef
UINT8 IP_ADDR[4];

/* Pointer to an IP Address */
typedef
UINT8 IP_ADDR_P[];

/* Definition of an IP packet header */
typedef struct
{
	UINT8		version_ihl;
	UINT8		service_type;
	UINT16		total_length;
	UINT16		identification;
	UINT16		flags_frag_offset;
	UINT8		ttl;
	UINT8		protocol;
	UINT16		checksum;
	IP_ADDR		source_addr;
	IP_ADDR		dest_addr;
	UINT8		options;	/* actually an array of undetermined length */
} ip_frame_hdr;

/* Macros for accessing an IP datagram.  */
#define IP_VERSION(a)	((a->version_ihl & 0x00F0) >> 4)
#define IP_IHL(a)		((a->version_ihl & 0x000F))
#define IP_SERVICE(a)	(a->service_type)
#define IP_LENGTH(a)	(a->total_length)
#define IP_IDENT(a)		(a->identification)
#define IP_FLAGS(a)		((a->flags_frag_offset & 0x0000E000) >> 13)
#define IP_FRAGMENT(a)	((a->flags_frag_offset & 0x00001FFF))
#define IP_TTL(a)		(a->ttl)
#define IP_PROTOCOL(a)	(a->protocol)
#define IP_CHKSUM(a)	(a->checksum)
#define IP_SRC(a)		(&a->source_addr[0])
#define IP_DEST(a)		(&a->dest_addr[0])
#define IP_OPTIONS(a)	(&a->options)
#define IP_DATA(a)		(&((UINT8 *)a)[IP_IHL(a) * 4])

/* Defined IP protocols */
#define IP_PROTO_ICMP	(1)
#define IP_PROTO_UDP	(17)

/****************************************************************/
typedef struct
{
	IP_ADDR			myip;
	IP_ADDR			gateway;
	IP_ADDR			netmask;
	IP_ADDR			broadcast;
	unsigned int	rx;
	unsigned int	rx_unsup;
	unsigned int	tx;
	unsigned int	err;
} PROTO_IP_INFO;

/****************************************************************/
int
ip_handler (NIF *, ip_frame_hdr *, int frame_length);

UINT16
ip_chksum (UINT16 *, int);

int
ip_send (
	NIF *,
	IP_ADDR_P,	/* destination IP */
	IP_ADDR_P,	/* source IP */
#if 0
	int,		/* identification */
#endif
	int,		/* protocol */
	UINT8 *,	/* data */
	int			/* data length */
	);

ip_frame_hdr *
ip_wait (NIF *, ip_frame_hdr *, int, int);

void
proto_ip_config (PROTO_IP_INFO *, IP_ADDR_P, IP_ADDR_P, IP_ADDR_P);

UINT8 *
proto_ip_get_pa (PROTO_IP_INFO *);

#endif 	/* _IP_H */
