/*
 * File:		dbug.h
 * Purpose:		Defintions global to dBUG.
 *
 * Notes:
 *
 * Date:		3-26-96
 *
 * Modifications:
 *
 */


#ifndef _DBUG_H
#define _DBUG_H

/*
 * Obtain Configuration Information
 */
#define DBUG_CONFIG
#include "src/dbug/board.h"
#undef  DBUG_CONFIG

/*
 * Obtain CPU basic types and other CPU information
 */
#include "cpu/cpu.h"

/*
 * Include board information and prototypes
 */
#include "board.h"
#include "src/dbug/board.h"

#ifdef TRUE
#undef TRUE
#endif
#define TRUE		(1)
#ifdef FALSE
#undef FALSE
#endif
#define FALSE		(0)
#ifdef NULL
#undef NULL
#endif
#define NULL		(0)

#define DEFAULT_MD_LINES	8

#define MAX_ARGS	10
#define MAX_BRKPTS	10
#define MAX_LINE	72

#define MAX_SYMBOL	250

#define BKPT_NONE	(0)
#define BKPT_PERM	(1)
#define BKPT_TEMP	(2)

#define SIZE_8		(8)
#define SIZE_16		(16)
#define SIZE_32		(32)
#define SIZE_64		(64)

/*
 * Where character I/O comes from during download.
 */
#define DL_SERIAL   (1)
#define DL_TFTP     (2)

#define DOWNLOAD_COFF   (1)
#define DOWNLOAD_SREC   (2)
#define DOWNLOAD_IMAGE  (3)
#define DOWNLOAD_ELF    (4)

#define BOARD_IO_SER	(1)
#define BOARD_IO_NET	(2)

/*
 * Linker defined symbol for start of user space
 */
extern char __USER_SPACE[];

/*
 * Global variable indicating radix or base of string conversions to
 * numbers.  String commonly used to print error message after failed
 * string conversion.
 */
extern int BASE;
extern const char INVALUE[];


/*
 * Prototypes for functions in history.c
 */

void
history_init (void);

char *
get_history_line (char *);


/*
 * Prototypes for functions in main.c
 */

void
pause (int *);

char *
get_line (char *);

NATURAL32
get_value (char *, int *, int);

void
match_cmd (char *);

int
make_argv (char *, char **);

void
main (void);


/*
 * Prototypes for functions in commands.c
 */

void
debugger_init (void);

void
breakpoint_init (void);

void
breakpoint_remove (ADDRESS);

void
breakpoint_add (ADDRESS, int);

int
breakpoint_install (ADDRESS);

int
breakpoint_deinstall (ADDRESS, int *);


/*
 * Prototypes for functions in stdlib.c
 */

int
strcasecmp (char *, char *);

int
strncasecmp (char *, char *, int);

unsigned long
strtoul (char *, char **, int);

#ifdef MYPRINTF
#define printf printD
#define sprintf sprintD

int
printD (const char *, ... );

int
sprintD (char *, const char *, ... );
#endif


/*
 * Prototypes for functions in symtab.c
 */

void
symtab_init (void);

void
symtab_add (char *, ADDRESS);

void
symtab_add_complete (char *, int);

void
symtab_add_incomplete (int, ADDRESS);

int
symtab_convert_string (char *, ADDRESS *);

int
symtab_convert_address (ADDRESS, char *);

void
download_coff (int);

void
download_srecord (int, int);

void
download_image (ADDRESS, int);


/*
 * Prototypes for dBUG commands
 */

void
bf (int, char **);

void
bm (int, char **);

void
breakpoint (int, char **);

void
bs (int, char **);

void
data (int, char **);

void
disasm (int, char **);

void
dl (int, char **);

void
dn (int, char **);

void
go (int, char **);

void
gotil (int, char **);

void
help (int, char **);

void
loopr (int, char **);

void
loopw (int, char **);

void
md (int, char **);

void
mm (int, char **);

void
reset (int, char **);

void
rd (int, char **);

void
rm (int, char **);

void
step (int, char **);

void
set (int, char **);

void
show (int, char **);

void
symbol (int, char **);

void
trace (int, char **);

void
vd (int, char **);

void
version (int, char **);


#endif	/* _DBUG_H */
